import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class redblue extends PApplet {

//REDBLUE THE GAME - 2 PLAYERS
//SHAURJYA BANERJEE 2018

//Values that work for num_sqaures - 4, 5, 8, 10, 20, 38
int num_sqaures;

PFont font;
int player_color;
int player2_color;
int color_map [][];

int game_states [][];
int r_min;
int r_max;
int player_x;
int player_y;
int player2_x;
int player2_y;
int grey_count;
int red_count;
int blue_count;
int red_perc;
int blue_perc;
int winner_state;
int intro_state;
int anim_state;
int anim_length;
int step_size;
int canvas_width  = 760;
int canvas_height = 760;
int x_offset;
int y_offset;
int win_step;
int end_time;
int first_state = 0;
int f_size;
int c_val;

//--------------------------------------------------------------------

public void setup()
{
  
  init();
}

public void draw()
{
  draw_maze();
  draw_characters(player_x, player_y, player2_x, player2_y);
  display_score();
  
  //Display intro sequence only on first play
  if (first_state == 0) {intro_sequence(60);}
}

//--------------------------------------------------------------------

//Function to initialize all variables at the start of a new game/round
public void init()
{
  frameRate(30);
  noStroke();
  textAlign(CENTER);
  
  int which_map = floor(random(1, 5));
  
  if      (which_map == 1) {num_sqaures = 4;}
  else if (which_map == 2) {num_sqaures = 5;}
  else if (which_map == 3) {num_sqaures = 8;}
  else if (which_map == 4) {num_sqaures = 10;}
  else if (which_map == 5) {num_sqaures = 20;}
 
  r_min = 125;
  r_max = 255;
  
  player_color  = color (255, 0, 0);
  player2_color = color (0, 0, 255);
  
  step_size    = 1;
  grey_count   = 0;
  red_count    = 0;
  blue_count   = 0;
  red_perc     = 0;
  blue_perc    = 0;
  winner_state = 0;
  intro_state  = 0;
  anim_state   = 0;
  anim_length  = 60;
  win_step     = 0;
  end_time     = 0;
  step_size    = 1;
  f_size       = 100;
  c_val        = 225;
  
  player_x = num_sqaures - 1;
  player_y = num_sqaures - 1;
  player2_x = 0;
  player2_y = 0;
  
  x_offset = canvas_width/num_sqaures;
  y_offset = canvas_height/num_sqaures;
  
  color_map   = new int [num_sqaures][num_sqaures];
  game_states = new int   [num_sqaures][num_sqaures];
  
  font = createFont("VCROSDMono", 20, true);
  generate_map();
  generate_grid();
}

//Function to handle timing for intro sequence
public void intro_sequence(int len)
{
  if (frameCount < len) {red_intro();}
  if (frameCount >=  len   && frameCount < len*2) {blue_intro();}
  if (frameCount >=  len*2 && frameCount < len*3) {rules1();}
  if (frameCount >=  len*3 && frameCount < len*4) {rules2();}
  if (frameCount >=  len*4 && frameCount < len*5) {go();}
  if (frameCount >= len* 5) {first_state = 1;}
}

//Function to handle the drawing of the score bar
public void display_score()
{
  int score_width = canvas_width/8;
  
  //Mask right side of screen
  fill(225);
  rect(num_sqaures * x_offset, 0, 200, height);
  
  //Draw red progress
  fill(map(sin(frameCount * 0.07f), -1, 1, 100, 255), 0, 0);
  rect(num_sqaures * x_offset + 10, height, score_width, -red_perc-6);
  
  //Draw blue progress
  fill(0, 0, (map(sin(frameCount * 0.07f), -1, 1, 100, 255)));
  rect(num_sqaures * x_offset + 10, 0, score_width, blue_perc+6);
  
  //Draw goal line
  fill(225);
  rect(num_sqaures * x_offset, (height/2)-5, 200, 10);
  
  check_winners(winner_state);
}

//This function handles the display of an end of game animation
// 0 = No Winners Yet
// 1 = Red Won
// 2 = Blue Won
public void check_winners (int who)
{
  textFont(font, 150);
  if (who != 0 && anim_state == 1)
  {
    if (who == 1)
    {
      fill(255 - c_val);
      text("RED", canvas_width/2, canvas_height/2 -70);
      text("WINS!", canvas_width/2, canvas_height/2 +70);
      
      //Restart the game if the animation is over
      if (frameCount > end_time) {init();}
    }
    else if (who == 2) 
    {
      fill(255 - c_val);
      text("BLUE", canvas_width/2, canvas_height/2 -70);
      text("WINS!", canvas_width/2, canvas_height/2 +70);
      
      //Restart the game if the animation is over
      if (frameCount > end_time) {init();}
    }
  }
}

//Function that handles a big chunk of the display duties 
//and scorekeeping mechanics while the game is running. 
public void draw_maze()
{
  for (int i = 0; i< num_sqaures; i++)
  {
    for (int j = 0; j < num_sqaures; j++)
    {
      float temp_x = i*x_offset;
      float temp_y = j*y_offset;
      float temp_red   = red  (color_map[i][j]);
      float temp_green = green(color_map[i][j]);
      float temp_blue  = blue (color_map[i][j]);

      color_map[i][j] = color (temp_red, temp_green, temp_blue);
      
      if (player_x == i && player_y == j)
      {color_map[i][j] = player_color;}
      
      if (player2_x == i && player2_y == j)
      {color_map[i][j] = player2_color;}
         
      //PLAYER 1
      fill(color_map[i][j]);
      rect(temp_x, temp_y, (temp_x+x_offset), (temp_y+y_offset));
      
      //PLAYER 2
      fill(color_map[i][j]);
      rect(temp_x, temp_y, (temp_x+x_offset), (temp_y+y_offset));
      
      //Check each block for its current color
      int check = get (i*x_offset, j*y_offset);
      
      float check_red   = red(check);
      float check_green = green(check);
      float check_blue  = blue(check);
      
      //Check for greyscale (denoted by 0)
      if (check_red == check_green && check_green == check_blue) {grey_count++; game_states[i][j] = 0;}
      //Check for red team (denoted by 1)
      if (check_blue == 0 && check_green == 0) {red_count ++; game_states[i][j] = 1;}
      //Check for blue team (denoted by 2)
      if (check_red == 0 && check_green == 0) {blue_count ++; game_states[i][j] = 2;}
    }
  }
  
  //Calculate red and blue percentages
  red_perc = ceil(map(red_count, 1, num_sqaures * num_sqaures, 1, canvas_height));
  blue_perc = ceil(map(blue_count, 1, num_sqaures * num_sqaures, 1, canvas_height));
  
  //Check if either player has majority possesion of the board
  if (red_perc > canvas_height/2) {winner_state = 1;}
  if (blue_perc > canvas_height/2) {winner_state = 2;}
  
  if ((winner_state == 1 || winner_state == 2) && anim_state == 0) 
  {
    anim_state = 1;
    win_step = frameCount;
    end_time = win_step + anim_length;
  }
  
  //Reset score counters
  grey_count = 0;
  red_count  = 0;
  blue_count = 0;
}


//Function to handle the display of each players current location
public void draw_characters(int x, int y, int x2, int y2)
{
  //PLAYER 1
  player_color = color(map(sin(frameCount * 0.07f), -1, 1, 100, 255), 0, 0);
  fill(player_color);
  rect(x*canvas_width/num_sqaures, y*canvas_height/num_sqaures, canvas_width/num_sqaures, canvas_height/num_sqaures);
  
  //PLAYER 2
  player2_color = color(0, 0, map(sin(frameCount * 0.07f), -1, 1, 100, 255));
  fill(player2_color);
  rect(x2*canvas_width/num_sqaures, y2*canvas_height/num_sqaures, canvas_width/num_sqaures, canvas_height/num_sqaures);
}

//Function to display the starting grid
public void generate_grid()
{
  float x_offset = canvas_width/num_sqaures;
  float y_offset = canvas_height/num_sqaures;
  
  for (int i = 0; i < num_sqaures; i++)
  {
    for (int j = 0; j< num_sqaures; j++)
    {
      float temp_x = i*x_offset;
      float temp_y = j*y_offset;
      fill(color_map[i][j]);
      rect(temp_x, temp_y, (temp_x+x_offset), (temp_y+y_offset));
    }
  }
}

//Function that populates our color map array 
//at the start of each new game/round
public void generate_map()
{
  for (int i = 0; i < num_sqaures; i++)
  {
    for (int j = 0; j < num_sqaures; j++)
    {
      color_map[i][j] = color(ceil(random(r_min, r_max)));
    }
  }  
}


//The keyReleased() function handles all player movement,
//including tracking each players current game co-ordinates,
//and restricting player movement on already captured squares
public void keyReleased() 
{
  //Calculate neighbour positions for player1
  int u = player_y - 1;
  int d = player_y + 1;
  int l = player_x - 1;
  int r = player_x + 1;
  
  //Check for edge cases for player1
  //(ie, when player1 is at either canvas border)
  if (u < 0) u = num_sqaures - 1;
  if (d > num_sqaures - 1) d = 0;
  if (l < 0) l = num_sqaures - 1;
  if (r > num_sqaures - 1) r = 0;
  
  //First, make sure no one has won yet, and that the intro is over
  if (winner_state == 0 && first_state != 0)
  {
    if (key == CODED) 
    {
      //These conditionals check keystrokes AND neighbour states 
      //before handling player1's game co-ordinates
      if (keyCode == UP && game_states[player_x][u] != 1)         {player_y -= step_size;} 
      else if (keyCode == DOWN && game_states[player_x][d] != 1)  {player_y += step_size;}
      else if (keyCode == LEFT && game_states[l][player_y] != 1)  {player_x -= step_size;} 
      else if (keyCode == RIGHT&& game_states[r][player_y] != 1)  {player_x += step_size;} 
    }
  }
  
  //Reset player location if player1 leaves the screen
  //The 'Snake' mechanic
  if (player_x < 0) {player_x = num_sqaures - 1;}
  else if (player_x > num_sqaures - 1) {player_x = 0;}
  if (player_y < 0) {player_y = num_sqaures - 1;}
  else if (player_y > num_sqaures - 1) {player_y = 0;}
  
  //Calculate neighbour positions for player2
  u = player2_y - 1;
  d = player2_y + 1;
  l = player2_x - 1;
  r = player2_x + 1;
  
  //Check for edge cases for player2
  //(ie, when player2 is at either canvas border)
  if (u < 0) u = num_sqaures - 1;
  if (d > num_sqaures - 1) d = 0;
  if (l < 0) l = num_sqaures - 1;
  if (r > num_sqaures - 1) r = 0;

  //Handle location counting to track player 2 movement
  //First make sure no one has won yet, and that the intro is over
  if (winner_state == 0 && first_state != 0)
  {
    //These conditionals check keystrokes AND neighbour states 
    //before handling player2's game co-ordinates
    if ((key == 'W' || key == 'w') && game_states[player2_x][u] != 2) {player2_y -= step_size;}
    if ((key == 'S' || key == 's') && game_states[player2_x][d] != 2) {player2_y += step_size;}
    if ((key == 'A' || key == 'a') && game_states[l][player2_y] != 2) {player2_x -= step_size;}
    if ((key == 'D' || key == 'd') && game_states[r][player2_y] != 2) {player2_x += step_size;}
  }
    
  //Reset player location if player2 leaves the screen
  //The 'Snake' mechanic
  if (player2_x < 0) {player2_x = num_sqaures - 1;}
  else if (player2_x > num_sqaures - 1) {player2_x = 0;}
  if (player2_y < 0) {player2_y = num_sqaures - 1;}
  else if (player2_y >num_sqaures - 1) {player2_y = 0;}
}

//Functions to display intro sequence -------------------------------
public void rules1()
{
  textFont(font, f_size);
  fill(255 - c_val);
  text("CAPTURE", canvas_width/2, canvas_height/2 - 90);
  text("ALL", canvas_width/2, canvas_height/2);
  text("SPACES", canvas_width/2, canvas_height/2 + 90);
}

public void rules2()
{
  textFont(font, f_size);
  fill(255 - c_val);
  text("YOU CANNOT", canvas_width/2, canvas_height/2 - 90);
  text("RECLAIM YOUR", canvas_width/2, canvas_height/2);
  text("OWN SPACES", canvas_width/2, canvas_height/2 + 90);
}

public void go()
{
  textFont(font, 150);
  fill(255 - c_val);
  text("GO!", canvas_width/2, canvas_height/2);
}

public void red_intro()
{
  textFont(font, f_size);
  fill(c_val, 0, 0);
  text("RED MOVES", canvas_width/2, canvas_height/2 - 90);
  text("USING", canvas_width/2, canvas_height/2);
  text("ARROW KEYS", canvas_width/2, canvas_height/2 + 90);
}

public void blue_intro()
{
  textFont(font, f_size);
  fill(0, 0, c_val);
  text("BLUE MOVES", canvas_width/2, canvas_height/2 - 90);
  text("USING", canvas_width/2, canvas_height/2);
  text("WASD KEYS", canvas_width/2, canvas_height/2 + 90);
}
  public void settings() {  size(875, 760); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "redblue" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
